@@ -1,14 +1,14 @@
 #' Data manipulation
 #' @description The function \code{dataprep} reshapes data from a long format to a ready-to-use format to be used directly in the function \code{ciregic}.
-#' @author Giorgos Bakoyannis, \email{gbakogia at iu dot edu}
 #' @author Jun Park, \email{jp84 at iu dot edu}
+#' @author Giorgos Bakoyannis, \email{gbakogia at iu dot edu}
 #' @param data a data frame that includes the variables named in the \code{ID}, \code{time}, \code{event}, and \code{z} arguments
 #' @param ID a variable indicating individuals' ID
 #' @param time a variable indicating observed time points
 #' @param event a vector of event indicator. If an observation is righ-censored, \code{event = 0}; otherwise, \code{event = 1} or \code{event = 2}, where \code{1} represents the first cause of failure, and \code{2} represents the second cause of failure. The current version of package only allows two causes of failure.
 #' @param Z a vector of variables indicating name of covariates
 #' @keywords dataprep
-#' @details The function \code{dataprep} provides a ready-to-use data format that can be directly used in the function \code{ciregic}. The returned data frame consists of \code{id}, \code{v}, \code{u}, \code{c}, and covariates as columns. The \code{v} and \code{u} indicate time window with the last observation time before the event and the first observation after the event. The \code{c} represents a type of event, for example, \code{c = 1} for the first cause of failure, \code{c = 2} for the second cause of failure, and \code{c = 0} for the right-censored. Individuals who have only one time record with right-censored event will be omitted because its time interval is \code{(0, Inf)}, and the lower bound \code{v} will be replaced by zero, for example \code{(0, v]}, if individuals are not right-censored and have only one time record.
+#' @details The function \code{dataprep} provides a ready-to-use data format that can be directly used in the function \code{ciregic}. The returned data frame consists of \code{id}, \code{v}, \code{u}, \code{c}, and covariates as columns. The \code{v} and \code{u} indicate time window with the last observation time before the event and the first observation after the event. The \code{c} represents a type of event, for example, \code{c = 1} for the first cause of failure, \code{c = 2} for the second cause of failure, and \code{c = 0} for the right-censored. For individuals having one time record with the event, the lower bound \code{v} will be replaced by zero, for example \code{(0, v]}. For individuals having one time record without the event, the upper bound \code{u} will be replaced by \code{Inf}, for example \code{(v, Inf]}.
 #' @return a data frame
 #' @examples
 #' library(intccr)
@@ -24,15 +24,23 @@ dataprep <- function(data, ID, time, event, Z) {
   Z <- unlist(strsplit(as.character(mcall$Z), " "))
   if(length(Z) > 1) Z <- Z[-1]
 
+  data <- data[order(data[, ID] & data[, time]), ]
+  tmiss <- sum(is.na(data[, time]))
+
+  if(tmiss > 0) {
+    print.df <- function(x) {
+      paste(capture.output(data[which(is.na(data[, time])), ]), collapse = "\n")
+    }
+    warning("The following records have missing visit times and will be discarded:\n\n", print.df(data))
+    data <- data[!is.na(data[, time]), ]
+  }
+
   uid <- sort(unique(data[, ID]))
   n <- length(uid)
   p <- length(Z)
   mZ <- data[colnames(data) %in% Z]
 
-  id <- rep(NA, n)
-  v <- rep(NA, n)
-  u <- rep(NA, n)
-  c <- rep(NA, n)
+  id <- v <- u <- c <- rep(NA, n)
   X <- matrix(data = NA, nrow = n, ncol = p, byrow = TRUE)
 
   for (i in 1:n){
@@ -48,6 +56,10 @@ dataprep <- function(data, ID, time, event, Z) {
         v[i] <- 0
         u[i] <- indt
         c[i] <- indc
+      } else {
+        v[i] <- indt
+        u[i] <- Inf
+        c[i] <- indc
       }
     } else {
       for (j in 1:length(indt)){
